# -*- coding: utf-8 -*-

addons   = {}  # Dictionary of loaded addons
pyaddons = []  # List of espython addons [tempfix for monkeypatch.py/createscriptlist]
events   = {}  # Events registered and their usage counts

import es, esc, cmds, parse, interface

selfref = __import__(__name__)
def eventlistener(ev): # Event listener
  name = ev['es_event']
  for addon in addons:
    addon = addons[addon]
    if addon.enabled and name in events:
      addon.doevent(name)
  esc.varcache.save()

class Addon(object):

  def __init__(self, scriptname, console=False):

    self.console = console
    self.condstack = []
    
    if self.console:
      self.condstack.append(None)

    else:
      self.scriptname = scriptname
      self.basename   = self.scriptname.split('/')[-1]
      self.scriptdir  = '%s/%s/' % (esc.ADDONDIR, scriptname)
      self.scriptfile = '%ses_%s.txt' % (self.scriptdir, self.basename)

  def load(self):

    script = open(self.scriptfile)
    lines = script.readlines()
    script.close()

    blocks = parse.splitblocks(lines)
    # TODO: Remove debug code. These come in handy for debugging parsing problems, though
    #import pprint
    #j = open("C:/temp/escdebug/" + self.basename + ".log", 'w')
    #j.writelines(pprint.pformat(blocks))
    #j.close()

    self.blocks, self.events = blocks['block'], blocks['event']

    for event in self.events:
      if event in events:
        events[event] += 1
      else:
        events[event] = 1
        es.addons.registerForEvent(selfref, event, eventlistener)

    self.enabled = True


  def unload(self):

    for event in self.events:
      events[event] -= 1
      if events[event] == 0:
        del events[event]
        es.addons.unregisterForEvent(esc._self, event)
    del self.blocks, self.events


  def enable(self):
    self.enabled = True
    

  def disable(self):
    self.enabled = False
    
    
  def scripttrace(self):
    return bool(interface.server_var('eventscripts_scripttrace'))


  def doblock(self, name, userargs=None, event=False):
    # TODO: Remove debug code. These come in handy for debugging parsing problems, though
    #import pprint
    #j = open("C:/temp/escdebug/" + self.basename + "---" + name.replace("/", "---") + ".log", 'w')
    #j.writelines(pprint.pformat(self.blocks[name]))
    #j.close()
    self.runblock(self.blocks[name], userargs, scripttrace=self.scripttrace())
    
  
  def doevent(self, name):
    if name in self.events:
      self.runblock(self.events[name], scripttrace=self.scripttrace())
    

  def runblock(self, code, userargs=None, scripttrace=None):
    self.condstack.append(None)
    for line in code:
      if type(line) is list:
        if not self.condstack[-1] is False:
          self.runblock(line, userargs, scripttrace)
        else:
          self.condstack[-1] = None 
      else:
        n, args, argv = line
        if scripttrace:
          es.dbgmsg(0, '[%s %s]%s%s %s' % (self.scriptname, n, " "*len(self.condstack),argv[0], args))
        self.genericcommand(list(argv), args, userargs)
    self.condstack.pop()

  def genericcommand(self, argv, args, userargs=None):
    (self.escommand if argv[0] in cmds.commands else self.command)(argv, args, userargs)
    
  def escommand(self, argv, args, userargs=None):
    
    cmdname = str(argv.pop(0))
    #es.dbgmsg(0, "#escommand: %s %s" % (cmdname, args))
    if self.alias(cmdname) or self.var(cmdname, args):
      return
    
    command = cmds.commands[cmdname]
    
    if cmdname in ('es', 'esnq', 'if', 'else') or (command.pre and cmdname[3] != 'x'):
      argv = parse.expand(argv, nq=cmdname=='esnq')
      args = None
    if not command.argsfrom is None:
      if args is None:
        args = parse.join(argv[command.argsfrom:])
      else:
        args = parse.argsfrom(args, command.argsfrom)
    #es.dbgmsg(0, "#escommand: %s -- %s" % (argv, userargs))
    command.run(self, argv, args, userargs)
    
  def command(self, argv, args, userargs=None):
    #es.dbgmsg(0, "#command: %s" % args)
    cmdname = str(argv.pop(0))
    if self.alias(cmdname) or self.var(cmdname, args):
      return
      
    if es.exists('command', cmdname):
      if args is None:
        args = parse.join(argv)
      getattr(es.server, cmdname)(args)
    else:
      es.dbgmsg(0, 'Unknown command "%s"' % cmdname)
        
  def clientcommand(self, argv, args, mapping=cmds.usercmds):
    
    cmdname = str(argv.pop(0))
    if args is None:
      args = parse.join(argv)
    block = mapping[cmdname]
    if block[0] in addons:
      addon = addons[block[0]]
      if block[1] in addon.blocks:
        argv.insert(0, cmdname)
        addon.doblock(block[1], userargs={'argv': argv, 'args': args, 'uid': int(es.getcmduserid())})
    else:
      esc.varcache.save()
      esc._pycmd_block[0] = '%s/%s' % block
      es.server._esc_firecmd(args)
    
  def alias(self, cmdname):
    if cmdname in esc.aliases:
      for args, argv in esc.aliases[cmdname]:
        self.genericcommand(list(argv), args)
      return True
    return False
    
  def var(self, cmdname, args):
    if es.exists('variable', cmdname):
      if argv:
        interface.setvar(cmdname, args)
      return True
    return False
