# -*- coding: utf-8 -*-

commands       = {}  # Eventscripts commandnames and objects
usercmds       = {}  # User commandnames and blocks
userclientcmds = {}  # User registered client commands and corresponding blocks
userchatcmds   = {}  # User registered chat commands and corresponding blocks

import inspect
import es, esc
from esc import addon, interface, monkeypatch, parse

class Command(object):

  def __init__(self,
      desc='',       # Command description
      pre=True,      # Use es and es_x prefix
      con=False,     # Send commandstring straight to console
      argsfrom=None, # Stop tokenizing after index
      alt=None,      # Alternative names for command
      strs=None      # Who the hell knows. Should probably lose this.
    ):
    self.desc = desc
    self.pre = pre
    self.con = con
    self.argsfrom = argsfrom
    if alt is None:
      self.alt = tuple()
    elif type(alt) != tuple:
      self.alt = (alt,)
    else:
      self.alt = alt
    if not strs in (None, True) and not type(strs) is tuple:
      strs = (strs,)
    self.strs = strs
    
  def __call__(self, method):
    self.method = method
    self.name = self.method.func_name
    if self.name[0] == '_':
      self.name = self.name[1:]
    ins = inspect.getargspec(self.method)
    args = ins[0]
    self.argv = 'argv' in args
    self.args = 'args' in args
    self.cond = 'cond' in args
    self.userargs = 'userargs' in args
    if self.pre:
      for prefix in ('es_', 'es_x'):
        commands[prefix+self.name] = self
    else:
      commands[self.name] = self
    for alt in self.alt:
      commands[alt] = self
    if self.con:
      self.args = True
      self.argv = False
      self.argsfrom = 0
      
    return self
    
  def run(self, addonref, argv, args, userargs=None):
  
    if self.con:
      esc.varcache.save()
      es.server.__getattr__('es_x%s' % self.name)(args)
    else:
      _args = {}
      if self.argv:
        if self.strs:
          if self.strs is True:
            args = map(str, argv)
          else:
            length = len(argv)
            for i in self.strs:
              if i < length:
                argv[i] = str(argv[i])
        _args['argv'] = argv
      if self.args:
        if args is None:
          args = parse.join(argv)
        _args['args'] = args
      if self.cond:
        _args['cond'] = cond
      if self.userargs:
        _args['userargs'] = userargs
      result = self.method(addonref, **_args)
      if not result is None:
        es.dbgmsg(0, result)
        
import addons, conditions, expansion, install, keygroups, maths, misc, output, players, reference, sounds, time, run, usercommands, vars, vectors, voodoo
    
@Command(desc='Alias a command.', pre=False, argsfrom=1)
def alias(addonref, argv, args, userargs=None):
  if not argv:
    es.dbgmsg(0, 'Current alias commands:\n')
    for alias in esc.aliases:
      es.dbgmsg(0, '%s : %s\n' % (alias, '; '.join('%s %s' % (i[1][0], i[0]) for i in esc.aliases[alias])))
  else:
    name = str(argv[0])
    esc.aliases[name] = [(parse.getargs(args), parse.tokenize(args)) for args in parse.splitcommands(args.replace('"', ''))]
    if not es.exists('command', name):
      monkeypatch._regcmd(name, 'esc/alias')
