import es, esc
from esc import interface, addon
from esc.cmds import Command

@Command(desc='Loads a script or lists all loaded scripts if no script is provided.', strs=True)
def load(addonref, argv):
  if argv:
    es.load(str(argv[0]))
  else:
    es.dbgmsg(0, '[Eventscripts] Loaded:\n')
    for addonname in sorted(addon.addons):
      es.dbgmsg(0, '[Eventscripts] [%sabled] %s' % ('en' if addon.addons[addonname].enabled else 'dis', addonname))
    for addonname in sorted(addon.pyaddons):
      es.dbgmsg(0, '[Eventscripts] [enabled] %s' % addonname)
    return

@Command(desc='Unloads a script that has been loaded.', strs=True)
def unload(addonref, argv, strs=True):
  if argv:
    es.unload(str(argv[0]))
  else:
    return 'Syntax: es_unload <scriptname>'
    
@Command(desc='Reloads a script that is loaded.', strs=True)
def _reload(addonref, argv):
  if argv:
    es.reload(str(argv[0]))
  else:
    return 'Syntax: es_reload <scriptname>'

@Command(desc='Enables a script that has been loaded.', strs=True)
def enable(addonref, argv):
  if argv:
    es.enable(str(argv[0]))
  else:
    return 'Syntax: es_enable <scriptname>'
  
@Command(desc='Disables a script that has been loaded.', strs=True)
def disable(addonref, argv):
  if argv:
    es.disable(str(argv[0]))
  else:
    return 'Syntax: es_disable <scriptname>'
  
@Command(desc='Executes a block.', strs=True)
def doblock(addonref, argv, userargs):
  if argv:
    name = argv[0].replace('\\', '/')
    while '//' in name:
      name = name.replace('//', '/')
    addonref, null, block = name.rpartition('/')
    if interface.blockexists(addonref, block):
      interface.doblock(addonref, block, userargs)
    else:
      esc.varcache.save()
      es.doblock(name)
  else:
    return 'Syntax: es_doblock <script>/<function>'
