import es, esc
from esc import interface, parse
from esc.cmds import Command

_if_operators = {
  'notlessthan': '<=',
  '=>': '<=',
  '=<': '>=',
  'notgreaterthan': '>=',
  '==': '==',
  '!=': '!=',
  'in': 'in',
  'notin': 'notin',
  '>': '>',
  '<': '<',
  '<=': '>=',
  '>=': '<=',
  'equalto': '==',
  '=': '==',
  'notequalto': '!=',
  '!==': '!=',
  'greaterthan': '>',
  'lessthan': '<',
}

_if_functions = {
  '==': str.__eq__,
  '!=': str.__ne__,
  'in': lambda x, y: y.__contains__(x),
  'notin': lambda x, y: not y.__contains__(x),
  '>': lambda x, y: x > y,
  '<': lambda x, y: x < y,
  '<=': lambda x, y: x <= y,
  '>=': lambda x, y: x >= y,
}

def getnum(value):
  if type(value) == str:
    return 0
  return value
  
@Command(desc='Allows you to conditionalize a script line.', pre=False, argsfrom=7, alt='es_xif')
def _if(addonref, argv, args, userargs):
  if len(argv) < 6 or (argv[0], argv[4]) != ('(', ')') or argv[5] not in ('then', 'do'):
    return 'Syntax: if (<value> <operator> <value>) <do>/<then> <commandstring>'
  op = argv[2]
  if not op in _if_operators:
    return 'Invalid conditional operator'
  op = _if_operators[op]
  one, two = argv[1], argv[3]
  if op in ('==', '!=', 'in', 'notin'):
    one, two = str(one), str(two)
  result = _if_functions[op](one, two)
  addonref.condstack[-1] = result
  if result and argv[5] == 'then':
    if len(argv) > 6:
      addonref.genericcommand(argv[6:], args, userargs=userargs)
  
  
@Command(desc='ifx <mode>(<arguments>) do\nNever supports expansion of variables.', pre=False)
def ifx(addonref, argv, args, userargs):
  if len(argv) < 5 or (argv[1], argv[3]) != ('(', ')') or argv[4] != 'do':
    return 'Syntax: ifx <operator>(<value>) <do>'
  op = argv[0].lower()
  if not op in ('true', 'false', 'parse'):
    return 'Invalid conditional operator'
  val = str(argv[2])
  if op in ('true', 'false'):
    # 0 0.00000          -- false .
    # 0 0.00000 0.00000  -- false
    # 0 0.00000 0        -- false .
    # 1 1.00000 1.00000  -- true
    # 2 2.10000 2.10000  -- true
    # 0 0.00000 Blah     -- true
    result = str(interface.server_var(val)) == '0' 
    if op == 'true':
      result = not result
  else:
    es.set('_tempcoreesc', 0)
    es.mathparse('_tempcoreesc', val)
    if '_tempcoreesc' in esc.varcache:
      del esc.varcache['_tempcoreesc']
    result = interface.server_var('_tempcoreesc') != '0'
  addonref.condstack[-1] = result
  
@Command(desc='else <any typical script line>\nExecutes the line if the previous \'if\' call returned false.', pre=False, alt='es_xelse', strs=0)
def _else(addonref, argv, args, userargs):
  if not argv:
    return 'Syntax: else <do>/<commandstring>'
  if not addonref.condstack[-1] is None:
    addonref.condstack[-1] = not addonref.condstack[-1]
  if argv[0] != 'do':
    addonref.genericcommand(argv, args, userargs)
    
@Command(desc='While loop', pre=False)
def _while(addonref, argv, userargs=None):
  if len(argv) < 2:
    return 'Syntax: while <condition> <commandstring>'
  argv = map(str, argv)
  cond = parse.expcompile(parse.tokenize(argv[0]))
  args = parse.argsfrom(argv[1], 1)
  argv = parse.tokenize(argv[1])
  while True:
    result = cond.expand()
    if not len(result) == 3 or not result[1] in _if_operators:
      return 'While: invalid condition'
    op = _if_operators[result[1]]
    one, two = result[0], result[2]
    if op in ('==', '!=', 'in', 'notin'):
      one, two = str(one), str(two)
    result = _if_functions[op](one, two)
    if not result:
      break
    addonref.genericcommand(list(argv), args, userargs=userargs)
