from __future__ import division
import math, re
import es, esc
from esc import interface
from esc.cmds import Command

_math_operators = {
  '+': lambda a, b: a + b,
  '-': lambda a, b: a - b,
  '*': lambda a, b: a * b,
  '/': lambda a, b: a / b,
  '^': lambda a, b: a ** b,
  '//': lambda a, b: a ** (1/b),
}
_math_operators['add'] = _math_operators['+']
_math_operators['subract'] = _math_operators['-']
_math_operators['multiply'] = _math_operators['*']
_math_operators['divide'] = _math_operators['/']
_math_operators['power'] = _math_operators['^']
_math_operators['root'] = _math_operators['//']

_math_functions = {
  'sin': math.sin,
  'cos': math.cos,
  'tan': math.tan,
  'asin': math.asin,
  'acos': math.acos,
  'atan': math.atan,  
  'abs': abs,
  'int': int,
  'float': float,
}

@Command(desc='EventScripts math functions, courtesy of Cr3V3TT3')
def math(addonref, argv):
  if len(argv) < 2:
    return 'Syntax: es_math <variable> <operator> [value]'
  var, op = argv[:2]
  if not es.exists('variable', var):
    return 'The var "%s" could not be found' % var
  one = interface.server_var(var)
  if op in _math_operators:
    if len(argv) < 3 or type(argv[2]) is str:
      return 'es_math: bad operation'
    r = _math_operators[op](one, argv[2])
  elif op in _math_functions:
    if op in ('int', 'float') and type(one) is str:
      one = 0
    r = _math_functions[op](one)
  else:
    return 'es_math: bad operation'
  interface.setvar(var, r)
  
@Command(con=True, desc='Syntax : es_rand <cvar> <minimum> <maximum>')
def rand(addonref, argv):
  pass

_alphanum_regex = re.compile('(?<=[^\w])(?!(a(bs|nd|vg)|a?cosh?|exp|if|l(n|og(10|2)?)|m(ax|in)|x?or|rint|s(ign|qrt|um)|a?sinh?|a?tanh?)[^\w]|$)[a-zA-Z_][\w]*', re.M)

_mp_cache = {}

def _expand(name):
  result = interface.server_var(name)
  if type(result) is str:
    return '0'
  return str(result)

_expand = interface.server_var
  
@Command(desc='Adds a say command that refers to a particular block.')
def mathparse(addonref, argv):
  if len(argv) < 2:
    return 'Syntax: es_mathparse <var> <math-expression>'
  es.set('_tempcore', 0)
  es.mathparse('_tempcore', argv[1])
  interface.setvar(argv[0], _expand('_tempcore'))

def mathparse(addonref, argv):
  if len(argv) < 2:
    return 'Syntax: es_mathparse <var> <math-expression>'
  es.set('_tempcore', 0)
  expr = argv[1]
  expression = ' %s' % expr
  if expr in _mp_cache:
    expr = _mp_cache[expr]
  else:
    #es.dbgmsg(0, 'INITIAL: %s' % expr)
    regex = list(_alphanum_regex.finditer(expression))
    for match in reversed(regex):
      start, end = match.span()
      expression = '%s%%s%s' % (expression[:start], expression[end:])
    expression = '%s %% (%s)' % (repr(expression), ', '.join('_expand(%s)' % repr(match.group(0)) for match in regex))
    #es.dbgmsg(0, 'RESULTING: %s' % expression)
    _mp_cache[expr] = expr = compile(expression, 'cmds.py', 'eval')
    
  eq = eval(expr)
  result = eval(eq)
  #es.dbgmsg(0, '%s = %s' % (eq, result))
  interface.setvar(argv[0], result)
