import es
import esc.cmds
from esc import interface
from esc.cmds import Command

@Command(desc='Adds a console command that refers to a particular block.', strs=True)
def regcmd(addonref, argv, strs=True):
  if len(argv) < 2:
    return 'Syntax: es_regcmd <command-name> <block-name> [description]'
  if es.exists('command', str(argv[0])):
    return 'es_regcmd: ERROR. Command %s already exists' % argv[0]
  es.regcmd(*argv[:3])
  
@Command(desc='Adds a client command that refers to a particular block.', strs=True)
def regclientcmd(addonref, argv):
  if len(argv) < 2:
    return 'Syntax: es_regclientcmd <command-name> <block-name> [description]'
  if es.exists('clientcommand', argv[0]):
    return 'es_regclientcmd: ERROR. Command %s already exists' % argv[0]
  es.regclientcmd(*argv[:3])
  
@Command(desc='Removes a client command that refers to a particular block.', strs=True)
def unregclientcmd(addonref, argv):
  if not argv:
    return 'Syntax: es_unregclientcmd <command>'
  if argv[0] not in esc.cmds.userclientcmds:
    return 'unregclientcmd: Did not find command: %s' % argv[0]
  es.unregclientcmd(argv[0])

@Command(desc='Adds a say command that refers to a particular block.', strs=True)
def regsaycmd(addonref, argv):
  if len(argv) < 2:
    return 'Syntax: es_regsaycmd <command-name> <block-name> [description]'
  if es.exists('saycommand', argv[0]):
    return 'es_regsaycmd: ERROR. Command %s already exists' % argv[0]
  es.regsaycmd(*argv[:3])
  
@Command(desc='Removes a say command that refers to a particular block.', strs=True)
def unregsaycmd(addonref, argv):
  if not argv:
    return 'Syntax: es_unregsaycmd <command>'
  if argv[0] not in esc.cmds.userchatcmds:
    return 'unregsaycmd: Did not find command: %s' % argv[0]
  es.unregsaycmd(argv[0])

@Command(desc='Gets the command parameter passed to the current ES console command.')
def getargv(addonref, argv, userargs):
  if len(argv) < 2:
    return 'Syntax: es_getargv <var> <index>'
  val = ''
  if userargs:
    i = argv[1]
    if type(i) is int:
      if i < len(userargs['argv']):
        val = userargs['argv'][i]
  interface.setvar(argv[0], val)
  
@Command(desc='Gets the commandstring passed to the current ES console command.')
def getargs(addonref, argv, userargs):
  if not argv:
    return 'Syntax: es_getargs <var>'
  interface.setvar(argv[0], userargs['args'] if userargs else '')
  
@Command(desc='Gets the count of parameters passed to the current ES console command.')
def getargc(addonref, argv, userargs):
  if not argv:
    return 'Syntax: es_getargc <var>'
  interface.setvar(argv[0], len(userargs['argv']) if userargs else 0)

@Command(desc='Gets the userid of the user that executed the command.')
def getcmduserid(addonref, argv, userargs):
  if not argv:
    return 'Syntax: es_getcmduserid <var>'
  elif userargs:
    interface.setvar(argv[0], userargs['id'])
  else:
    interface.setvar(argv[0], 0)
