import es
from esc import interface
from esc.cmds import Command


@Command(desc='Adds a new server/global variable.', alt='setinfo')
def _set(addonref, argv):
  if len(argv) < 2:
    return 'Syntax: es_set <key> <value> [description]'
  interface.setvar(create=True, *argv[:3])
  
@Command(desc='Makes a console variable public such that changes to it are announced to clients.')
def makepublic(addonref, argv):
  if not argv:
    return 'Syntax: es_makepublic <variable>'
  name = str(argv[0])
  if not es.exists('variable', name):
    return 'The var "%s" could not be found' % name
  es.makepublic(name)
  
@Command(desc='Reads the server variable referenced by varname2 and copies it into the variable referenced by varname.', strs=True)
def copy(addonref, argv):
  if len(argv) < 2:
    return 'Syntax: es_copy <varname> <varname2>'
  if not es.exists('variable', argv[1]):
    return 'The var "%s" could not be found' % argv[1]
  interface.setvar(argv[0], interface.server_var(argv[1]))

@Command(desc='Allows you to format a string by filling in a list of strings into a format string.', strs=True)
def _format(addonref, argv):
  if len(argv) < 2:
    return 'Syntax: es_format <variable> <format-string> [token1] [token2] [...] [tokenN]'
  formatstring = argv[1]
  for n, token in enumerate(argv[2:11]):
    formatstring = formatstring.replace('%%%d' % (n+1), str(token))
  interface.setvar(argv[0], formatstring, create=True)
  
@Command(desc='Allows you to format a string by filling in a list of strings into a format string.')
def formatv(addonref, argv):
  if len(argv) < 2:
    return 'Syntax: es_formatv <variable> <format-string> [token1] [token2] [...] [tokenN]'
  formatstring = str(argv[1])
  for n, token in enumerate(argv[2:11]):
    if '%%%d' % (n+1) in formatstring:
      formatstring = formatstring.replace('%%%d' % (n+1), str(interface.server_var(token)))
  interface.setvar(argv[0], formatstring, create=True)

@Command(con=True, desc='Allows you to format a string by filling in a list of strings into a format string.')
def formatqv(addonref, argv):
  pass

@Command(desc='EventScripts string functions, experimental support originally written by Cr3V3TT3')
def string(addonref, argv):
  if len(argv) < 3:
    return 'Syntax: es_string <variable> <operator> <value1> [value2]'
  if not es.exists('variable', argv[0]):
    return 'The var "%s" could not be found' % argv[0]
  val = str(interface.server_var(argv[0]))
  op = argv[1]
  one = argv[2]
  if op == 'replacev':
    one = str(interface.server_var(one))
    op = 'replace'
  if op == 'replace':
    r = ''
    if len(argv) > 3:
      r = str(argv[3])
    val = val.replace(argv[2], r)
  elif op == 'section':
    max = None
    if len(argv) > 3:
      max = argv[3]
      if type(max) != int:
        max = None
    if type(one) != int:
      one = 0
      val = val[min:max] 
  interface.setvar(argv[0], val)
    
  
@Command(desc='Set a variable to a particular token in the string.\nSyntax: es_xtoken <variable> <string> [token#] [tokenchar]')
def token(addonref, argv):
  if len(argv) < 3:
    return 'Syntax: es_token <variable> <string> <token#> [tokenchar]'
  sep = ' '
  if len(argv) > 3:
    sep = str(argv[3])
  n = argv[2]
  if not type(n) is int:
    n = 0
  tokens = argv[1].split(sep)
  if n:
    interface.setvar(argv[0], len(tokens))
  else:
    if len(tokens) > n:
      return 'The specified token number doesn\'t exist'
    interface.setvar(argv[0], tokens[n-1])
  
@Command(desc='Forces a variable to a particular value')
def forcevalue(addonref, argv):
  if len(argv) < 2:
    return 'Syntax: es_forcevalue <key> <value>'
  es.forcevalue(str(argv[0]), str(argv[1]))
  val = str(es.server_var[argv[0]])
  esc.varcache[argv[0]] = argv[1]

@Command(con=True, desc='Adds or removes the cheat flag from a command or variable. (EXPERIMENTAL/UNSUPPORTED)')
def flags(addonref, argv):
  pass

@Command(con=True, desc='Compares two strings.')
def strcmp(addonref, argv):
  pass

@Command(con=True, desc='Returns the length of a string.')
def strlen(addonref, argv):
  pass
  
@Command(con=True, desc='Sets a convar value for a fake client.')
def botsetvalue(addonref, argv):
  pass
  
@Command(desc='Various regular expression commands.')
def regex(addonref, argv):
  if not argv or not argv[0] in ('match', 'search', 'matchformat'):
    return 'es_regex: incorrect operator'
  if len(argv) < 4:
    return 'Syntax: es_regex <operator> <variable> <expression> <string>'
  try:
    regex = re.compile(argv[2])
  except:
    return 'Invalid regex'
  string = str(argv[3])
  op = argv[0]
  if op == 'match':
    match = regex.match(string)
    result = -1
    if match:
      result = match.span()[1]
  elif op == 'matchformat':
    match = regex.match(string)
    result = None
    if match:
      result = argv[4]
      match = match.groups()
      length = len(match)
      for i in xrange(1, 10):
        r = ''
        if i <= length:
          r = match[i-1]
        result = result.replace('%%%s' % i, r)
  elif op == 'search':
    length = len(argv)
    start = argv[5] if length > 5 and type(argv[5]) is int else 0
    length = argv[6] if length > 6 and type(argv[6]) is int else None
    if not length is None:
      length += start
    match = regex.match(string[start:length])
    result = -1
    if match:
      result = start + match.span()[0]
  interface.setvar(argv[1], result)
  return
  
  op = argv[0]
  if op == 'match':
    if len(argv) < 5:
      return 'Syntax: es_regex match <variable> <expression> <string>'
  elif op == 'search':
    if len(argv) < 5:
      return 'Syntax: es_regex search <variable> <expression> <string> [start] [range]'
    stop = 6
    if len(argv) > 5:
      argv[5] = getnum(argv[5], True)
      if len(argv) > 6:
        argv[6] = getnum(argv[6], True)
  if op == 'matchformat':
    if len(argv) < 5:
      return 'Syntax: es_regex matchformat <variable> <expression> <string> <formatstring> '
    stop = 5
  var = argv.pop(1)
  interface.setvar(var, es.regex(*argv[:stop]))
