# -*- coding: utf-8 -*-

import os
import es, esc, addon, parse

def loadaddon(addonname):
  addonname = parse.cleanpath(addonname)
  if addonname in addon.addons or addonname in addon.pyaddons:
    es.dbgmsg(0, '[EventScripts] Load script failed. It might already be loaded, try to es_unload it first')
    return False
  ad = addon.Addon(addonname)
  if not os.path.exists(ad.scriptfile):
    es.dbgmsg(0, 'Could not open script for %s' % ad.scriptname)
    return False
  ad.load()
  addon.addons[addonname] = ad
  loadblock = 'load'
  if loadblock in ad.blocks:
    ad.doblock(loadblock)
  es.dbgmsg(1 if "corelib" in ad.scriptname else 0, 'Loaded %s' % ad.scriptname)
    
def unloadaddon(addonname):
  addonname = parse.cleanpath(addonname)
  es.dbgmsg(0, 'Unloading %s...' % addonname)
  if not addonname in addon.addons:
    es.dbgmsg(0, '%s was not loaded' % addonname)
    return False
  ad = addon.addons[addonname]
  unloadblock = 'unload'
  if unloadblock in ad.blocks:
    ad.doblock(unloadblock)
  unloadblock = '_unload'
  if unloadblock in ad.blocks:
    ad.doblock(unloadblock)
  addon.addons[addonname].unload()
  del addon.addons[addonname]
  es.dbgmsg(0, '%s has been unloaded' % addonname)
  return True
    
def reloadaddon(addonname):
  unloadaddon(addonname)
  return loadaddon(addonname)
    
def enableaddon(addonname):
  addonname = parse.cleanpath(addonname)
  es.dbgmsg(0, 'Enabling %s...' % addonname)
  if addonname in addon.addons:
    addon.addons[addonname].enable()
    return True
  es.dbgmsg(0, '%s is not loaded' % addonname)
  return False
    
def disableaddon(addonname):
  addonname = parse.cleanpath(addonname)
  es.dbgmsg(0, 'Disabling %s...' % addonname)
  if addonname in addon.addons:
    addon.addons[addonname].disable()
    return True
  es.dbgmsg(0, '%s is not loaded' % addonname)
  return False
      
def doblock(addonname, block, userargs=None):
  es.dbgmsg(1, "interface.doblock %s %s" % (addonname, block))
  addonname = parse.cleanpath(addonname)
  if not blockexists(addonname, block):
    return False
  addon.addons[addonname].doblock(block, userargs)
  return True
    
def addonexists(addonname):
  return parse.cleanpath(addonname) in addon.addons
    
def blockexists(addonname, block):
  addonname = parse.cleanpath(addonname)
  return addonexists(addonname) and block in addon.addons[addonname].blocks
    
def eventexists(addonname, event):
  addonname = parse.cleanpath(addonname)
  return addonexists(addonname) and event in addon.addons[addonname].events

def setvar(name, value, desc=None, create=False):
  name = str(name)
  #TODO: Reconsider adding varcache support back
  if create or es.exists('variable', name) or server_var('eventscripts_autocreate') != '0':
    #esc.varcache[name] = value
    #if name[0] != "_":
    if not desc is None:
      es.set(name, value, desc)
    else:
      es.set(name, value)

sv = es.getString
def server_var(name):
  name = str(name)
  # TODO: consider adding varcache support back
  if es.exists('variable', name):
    value = parse.coerce(sv(name))
  else:
    value = 0
  return value

ev = es.getEventInfo
def event_var(name):
  return parse.coerce(ev(str(name)))
