# -*- coding: utf-8 -*-

import os, re, sys
import es, esc, cmds, addon, interface

def _load(addonname):

  if os.path.exists('%s/%s/%s.py' % (esc.ADDONDIR, addonname, addonname.split('/')[-1])):
    _load(addonname)
    addon.pyaddons.append(addonname)
  else:
    interface.loadaddon(addonname)
    
es.load, _load = _load, es.load

def _unload(addonname):
  if addonname in addon.pyaddons:
    _unload(addonname)
    addon.pyaddons.remove(addonname)
  else:
    interface.unloadaddon(addonname)
    
es.unload, _unload = _unload, es.unload

def _reload(addonname):
  es.unload(addonname)
  es.load(addonname)
  
es.reload, _reload = _reload, es.reload

def _enable(addonname):
  _enable(addonname) if addonname in addon.pyaddons else interface.enableaddon(addonname)
    
es.enable, _enable = _enable, es.enable

def _disable(addonname):
  _disable(addonname) if addonname in addon.pyaddons else interface.disableaddon(addonname)
    
es.disable, _disable = _disable, es.disable

def _doblock(name, userargs=None):
  es.dbgmsg(1, "_doblock %s" %name)
  name = name.replace('\\', '/')
  while '//' in name:
    name = name.replace('//', '/')
  addonname, null, block = name.rpartition('/')
  if interface.blockexists(addonname, block):
    interface.doblock(addonname, block, userargs)
    esc.varcache.save()
  else:
    _doblock(name)

es.doblock, _doblock = _doblock, es.doblock

def _exists(attribute, arg, *arguments):
  if attribute == 'script':
    if arg in addon.pyaddons:
      return True
    else:
      return interface.addonexists(arg)
  elif attribute == 'block':
    if not '/' in arg:
      return False
    addonname, null, block = arg.rpartition('/')
    if addonname in addon.pyaddons:
      addonname = addonname.replace('/', '.')
      addonname = getattr(sys.modules[addonname.replace('/', '.')], (addonname.split('/')[-1]))
      return block in dir(addonname)
    else:
      return interface.blockexists(addonname, block)
  else:
    return _exists(attribute, arg, *arguments)
    
es.exists, _exists = _exists, es.exists

def _regcmd(command, block, desc=None):
  if not es.exists('command', command):
    block = block.replace('\\', '/')
    while '//' in block:
      block = block.replace('//', '/')    
    addonname, null, block = block.rpartition('/')
    cmds.usercmds[command] = (addonname, block)
    _regcmd(command, 'esc/usercommand', desc)
  
es.regcmd, _regcmd = _regcmd, es.regcmd

def _regclientcmd(command, block, desc=None):
  if not es.exists('clientcommand', command):
    block = block.replace('\\', '/')
    while '//' in block:
      block = block.replace('//', '/')    
    addonname, null, block = block.rpartition('/')
    cmds.userclientcmds[command] = (addonname, block)
    _regclientcmd(command, 'esc/clientcommand', desc)
  
es.regclientcmd, _regclientcmd = _regclientcmd, es.regclientcmd

def _unregclientcmd(command):
  if command in cmds.userclientcmds:
    del cmds.userclientcmds[command]
    _unregclientcmd(command)
  
es.unregclientcmd, _unregclientcmd = _unregclientcmd, es.unregclientcmd

def _regsaycmd(command, block, desc=None):
  if not es.exists('saycommand', command):
    block = block.replace('\\', '/')
    while '//' in block:
      block = block.replace('//', '/')    
    addonname, null, block = block.rpartition('/')    
    cmds.userchatcmds[command] = (addonname, block)
    _regsaycmd(command, 'esc/saycommand', desc)
  
es.regsaycmd, _regsaycmd = _regsaycmd, es.regsaycmd

def _unregsaycmd(command):
  if command in cmds.userchatcmds:
    del cmds.userchatcmds[command]
    _unregsaycmd(command)
  
es.unregsaycmd, _unregsaycmd = _unregsaycmd, es.unregsaycmd

def _createscriptlist(match=None):
  if match is None:
    return dict([(script, {'type': 'py', 'status': 'enabled'}) for script in addon.pyaddons]+[(script, {'type': 'txt', 'status': 'enabled' if addon.addons[script].enabled else 'disabled'}) for script in addon.addons])
  if es.exists('script', match):
    a = {match: {}}
    a[match]['type'] = 'py' if match in addon.pyaddons else 'txt'
    a[match]['status'] = 'enabled' if match in addon.pyaddons else ('enabled' if addon.addons[match].enabled else 'disabled')
    return a
  return {}
  
es.createscriptlist, _createscriptlist = _createscriptlist, es.createscriptlist
