# -*- coding: utf-8 -*-

import re
import esc, interface, parse
import es

# Split addon into hierarchical blocks and events, with separated and tokenized commands. Mostly voodoo, but written to be fast and non mem intensive.

def splitblocks(lines, blocks=None, currentblock=None, level=0, addn=1):
  if blocks is None:
    blocks = {'block': {}, 'event': {}}
  n = 0
  end = 0
  for n, line in enumerate(lines):
    # skip the lines if they were part of the last block
    if end > 0:
      end -= 1
      addn += 1
      continue 
    lineno = n + addn
    line = line.strip()
    #es.dbgmsg(0, "parsing: %d %s%s" % (n, " "*2*level, line))
    if not line:
      continue
    if currentblock is None:
      match = re.match('^(block|event) +([^ ]+)', line)
      if match and len(lines) > n+1 and lines[n+1].startswith('{'):
        blocktype, name = match.groups()
        currentblock = blocks[blocktype][name] = []
        del lines[n]
        addn += 1
    elif line.startswith('{'):
      #es.dbgmsg(0, "parsed startblock at %d" % lineno)
      newblock = []
      end = splitblocks(lines[n+1:], blocks, newblock, level+1, addn=lineno+1)
      #es.dbgmsg(0, "  deleting lines:%d through %d" % (addn+n+1, addn+n+end+1))
      #for j in lines[n+1:n+1+end]:
      #  es.dbgmsg(0, "   deleting:%s" % j)
      #del lines[n+1:n+end+1]
      #addn += end
      addn += 1
      if newblock:
        currentblock.append(newblock)
    elif line.startswith('}'):
      #es.dbgmsg(0, "parsed endblock at %d size: %d" % (lineno, len(currentblock))) 
      if level:
        return n + 1
      currentblock = None
    else:
      #es.dbgmsg(0, "parsed codestuff at %d" % lineno)
      for line in splitcommands(remcomment(line)):
        currentblock.append((lineno, getargs(line), tokenize(line)))
  if not level:
    return blocks
  return n
  
  
# Split line into tokens a la source console
    
def tokenize(line):
  if line.count('"') % 2 == 1:
    line = '%s"' % line.strip()
  tokens = _parsetokens_regex.findall(line)
  return [parse.coerce(match.replace('"', '')) for match in _parsetokens_regex.findall(line)]
  
  
# Joins separated tokens into commandstring, complete with quote marks where necessary to avoid future tokenization

_tokenchars_regex = re.compile('[{}()\':; ]')
def join(tokens):
  return ' '.join(('"%s"' if not token or (_tokenchars_regex.search(token) and (len(token) > 1 or token == ';')) else '%s') % token for token in map(str, tokens))
  
  
# Get arguments as a string

_parsetokens_regex = re.compile('"[^"]*"|[{}()\':;]|[^"{}()\':; ]+')
def getargs(line):
  return line[_parsetokens_regex.search(line).span()[1]:].strip()
  
  
# Get arguments as a string from a given index

def argsfrom(line, index):
  while index:
    index -= 1
    match = _parsetokens_regex.search(line)
    if not match:
      return ''
    line = line[match.span()[1]:]
  return line.strip()
  
  
# Split any seperate; commands; on a single line

_splitcmds = re.compile('(?:"[^"]*"|[^";]+)+')
def splitcommands(line):
  return filter(None, map(str.strip, _splitcmds.findall(line)))


# Remove any //comments from a command line

def remcomment(line): # Returns line minus any //comments
  if '//' in line:
    sections = line.split('//')
    for n, section in enumerate(sections):
      if section.count('"') % 2 == 0:
        line = '//'.join(sections[:n+1])
        break
  return line.strip()
  
  
# Expand any variables, given tokens

def expand(tokens, nq=False):
  if 'server_var' in tokens or 'event_var' in tokens:
    for n, token in reversed(list(enumerate(tokens[:-3]))):
      if token in ('server_var', 'event_var') and (tokens[n+1], tokens[n+3]) == ('(', ')'):
				value = [getattr(interface, token)(tokens[n+2])]
				if type(value[0]) is str:
					if not ';' in value[0] and (nq or not re.search('[;%s]' % re.escape(interface.server_var('eventscripts_escapechars')), value[0])):
						value = tokenize(value[0])
				tokens[n:n+4] = value  
  return tokens
  
  
# Coerces strings, ints and floats into their equivalent python types  

_isfloat_regex = re.compile('^-?\d*\.\d+$')
def coerce(val):
  if not type(val) is str or not val or not (val[0].isdigit() or val[0] == '.'):
    return val
  if val.isdigit():
    return int(val)
  if _isfloat_regex.match(val):
    return float(val)
  return val
  
  
# Cleans up addon paths that look//like\this

def cleanpath(path):
  path = path.replace('\\', '/')
  while '//' in path:
    path = path.replace('//', '/')
  return path
  
  
# Creates a compiled commandstring object for very fast expansion, for JIT compilation of frequently used lines [e.g. using "while"]

class expcompile(object):
  
  def __init__(self, tokens, nq=False):
    for n, token in reversed(list(enumerate(tokens[:-3]))):
      if token in ('server_var', 'event_var'):
        if (tokens[n+1], tokens[n+3]) == ('(', ')'):
          tokens[n:n+4] = [self.sv(tokens[n+2])]
    self.tokens = tokens
    self.varindex = [n for n, token in enumerate(self.tokens) if type(token) not in (str, int, float)]
    self.regex = re.compile('[%s]' % re.escape(interface.server_var('eventscripts_escapechars')))
    self.nq = nq

  def expand(self):

    return list(self._expand())

  def _expand(self):

    for n, token in enumerate(self.tokens):
      if n in self.varindex:
        token = token.expand()
        if type(token) is str:
          if not ';' in token and (self.nq or not self.regex.search(token)):
            for token in tokenize(token):
              yield token
        else:
          yield token
      else:
        yield token

  class sv(object):
      
    def __init__(self, name, event=False):
      self.name = name
      self.func = getattr(interface, '%s_var' % ('event' if event else 'server'))

    def expand(self):
      return self.func(str(self.name))
